@@ -27,6 +27,7 @@
 #include "outbound.h"
 #include "cfgfiles.h"
 #include "ignore.h"
+#include "server.h"
 #include "servlist.h"
 #include "modes.h"
 #include "notify.h"
@@ -163,6 +164,15 @@ plugin_free (xchat_plugin *pl, int do_deinit, int allow_refuse)
 #endif
 
 xit:
+	if (pl->free_strings)
+	{
+		if (pl->name)
+			free (pl->name);
+		if (pl->desc)
+			free (pl->desc);
+		if (pl->version)
+			free (pl->version);
+	}
 	if (pl->filename)
 		free ((char *)pl->filename);
 	free (pl);
@@ -179,7 +189,7 @@ xit:
 static xchat_plugin *
 plugin_list_add (xchat_context *ctx, char *filename, const char *name,
 					  const char *desc, const char *version, void *handle,
-					  void *deinit_func, int fake)
+					  void *deinit_func, int fake, int free_strings)
 {
 	xchat_plugin *pl;
 
@@ -192,6 +202,7 @@ plugin_list_add (xchat_context *ctx, char *filename, const char *name,
 	pl->version = (char *)version;
 	pl->deinit_callback = deinit_func;
 	pl->fake = fake;
+	pl->free_strings = free_strings;	/* free() name,desc,version? */
 
 	plugin_list = g_slist_prepend (plugin_list, pl);
 
@@ -226,7 +237,7 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 		file = strdup (filename);
 
 	pl = plugin_list_add (sess, file, file, NULL, NULL, handle, deinit_func,
-								 fake);
+								 fake, FALSE);
 
 	if (!fake)
 	{
@@ -905,7 +916,7 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 	while (slist)
 	{
 		serv = slist->data;
-		netname = get_network (serv->front_session, TRUE);
+		netname = server_get_network (serv, TRUE);
 
 		if (servname == NULL ||
 			 rfc_casecmp (servname, serv->servername) == 0 ||
@@ -977,7 +988,7 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		return XCHATLIBDIR;
 
 	case 0x6de15a2e:	/* network */
-		return get_network (sess, FALSE);
+		return server_get_network (sess->server, FALSE);
 
 	case 0x339763: /* nick */
 		return sess->server->nick;
@@ -1238,7 +1249,7 @@ xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0x38b735af: /* context */
 			return data;	/* this is a session * */
 		case 0x6de15a2e: /* network */
-			return get_network ((session *)data, FALSE);
+			return server_get_network (((session *)data)->server, FALSE);
 		case 0x8455e723: /* nickprefixes */
 			return ((session *)data)->server->nick_prefixes;
 		case 0x829689ad: /* nickmodes */
@@ -1384,8 +1395,8 @@ xchat_plugingui_add (xchat_plugin *ph, const char *filename,
 							const char *version, char *reserved)
 {
 #ifdef USE_PLUGIN
-	ph = plugin_list_add (NULL, strdup (filename), name, desc, version, NULL,
-								 NULL, TRUE);
+	ph = plugin_list_add (NULL, strdup (filename), strdup (name), strdup (desc),
+								 strdup (version), NULL, NULL, TRUE, TRUE);
 	fe_pluginlist_update ();
 #endif
 
