@@ -228,8 +228,9 @@ void
 tcp_sendf (server *serv, char *fmt, ...)
 {
 	va_list args;
-	/* keep this buffer in BSS */
-	static char send_buf[520];	/* good code hey (no it's not overflowable) */
+	/* keep this buffer in BSS. Converting UTF-8 to ISO-8895-x might make the
+      string shorter, so allow alot more than 512 for now. */
+	static char send_buf[1540];	/* good code hey (no it's not overflowable) */
 	int len;
 
 	va_start (args, fmt);
@@ -306,6 +307,11 @@ server_inline (server *serv, char *line, int len)
 			conv_line[len] = 0;
 			conv_len = len;
 
+			/* if CP1255, convert it with the NUL terminator.
+				Works around SF bug #1122089 */
+			if (serv->using_cp1255)
+				conv_len++;
+
 			do
 			{
 				err = NULL;
@@ -333,6 +339,8 @@ server_inline (server *serv, char *line, int len)
 			{
 				line = utf_line_allocated;
 				len = utf_len;
+				if (serv->using_cp1255 && len > 0)
+					len--;
 			}
 			else
 			{
@@ -414,7 +422,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 				serv->linebuf[serv->pos] = 0;
 				if (prefs.stripcolor)
 				{
-					temp = strip_color (serv->linebuf);
+					temp = strip_color (serv->linebuf, -1, 1, 1);
 					server_inline (serv, temp, strlen (temp));
 					free (temp);
 				} else
@@ -440,24 +448,8 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 static void
 server_connected (server * serv)
 {
-	char outbuf[512];
-
-	serv->ping_recv = time (0);
-
-#ifdef WIN32
-	if (prefs.identd)
-		identd_start ((serv->username) ? serv->username : prefs.username);
-#else
-	sprintf (outbuf, "%s/auth/xchat_auth", g_get_home_dir ());
-	if (access (outbuf, X_OK) == 0)
-	{
-		sprintf (outbuf, "exec -d %s/auth/xchat_auth %s", g_get_home_dir (),
-					prefs.username);
-		handle_command (serv->server_session, outbuf, FALSE);
-	}
-#endif
-
 	prefs.wait_on_exit = TRUE;
+	serv->ping_recv = time (0);
 	serv->connected = TRUE;
 	set_nonblocking (serv->sok);
 	serv->iotag = fe_input_add (serv->sok, FIA_READ|FIA_EX, server_read, serv);
@@ -465,9 +457,21 @@ server_connected (server * serv)
 	{
 		EMIT_SIGNAL (XP_TE_CONNECTED, serv->server_session, NULL, NULL, NULL,
 						 NULL, 0);
-		serv->p_login (serv,
-							(serv->username) ? serv->username : prefs.username,
-							(serv->realname) ? serv->realname : prefs.realname);
+		if (serv->network)
+		{
+			serv->p_login (serv,
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->user)) ?
+								(((ircnet *)serv->network)->user) :
+								prefs.username,
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->real)) ?
+								(((ircnet *)serv->network)->real) :
+								prefs.realname);
+		} else
+		{
+			serv->p_login (serv, prefs.username, prefs.realname);
+		}
 	} else
 	{
 		EMIT_SIGNAL (XP_TE_SERVERCONNECTED, serv->server_session, NULL, NULL,
@@ -811,7 +815,7 @@ waitline2 (GIOChannel *source, char *buf, int bufsize)
 
 #else
 
-#define waitline2(source,buf,size) waitline(serv->childread,buf,size)
+#define waitline2(source,buf,size) waitline(serv->childread,buf,size,0)
 
 #endif
 
@@ -866,6 +870,10 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 
 	switch (tbuf[0])
 	{
+	case '0':	/* print some text */
+		waitline2 (source, tbuf, sizeof tbuf);
+		PrintText (serv->server_session, tbuf);
+		break;
 	case '1':						  /* unknown host */
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
@@ -897,6 +905,26 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 		waitline2 (source, ip, sizeof ip);
 		waitline2 (source, outbuf, sizeof outbuf);
 		EMIT_SIGNAL (XP_TE_CONNECT, sess, host, ip, outbuf, NULL, 0);
+#ifdef WIN32
+		if (prefs.identd)
+		{
+			if (serv->network)
+				identd_start ((((ircnet *)serv->network)->user) ?
+									(((ircnet *)serv->network)->user) :
+									prefs.username);
+			else
+				identd_start (prefs.username);
+		}
+#else
+		snprintf (outbuf, sizeof (outbuf), "%s/auth/xchat_auth",
+					 g_get_home_dir ());
+		if (access (outbuf, X_OK) == 0)
+		{
+			snprintf (outbuf, sizeof (outbuf), "exec -d %s/auth/xchat_auth %s",
+						 g_get_home_dir (), prefs.username);
+			handle_command (serv->server_session, outbuf, FALSE);
+		}
+#endif
 		break;
 	case '4':						  /* success */
 		waitline2 (source, tbuf, sizeof (tbuf));
@@ -993,11 +1021,6 @@ server_disconnect (session * sess, int sendquit, int err)
 	/* send our QUIT reason */
 	if (sendquit && serv->connected)
 	{
-		if (serv->eom_cmd)
-		{
-			free (serv->eom_cmd);
-			serv->eom_cmd = NULL;
-		}
 		server_sendquit (sess);
 	}
 
@@ -1034,11 +1057,6 @@ server_disconnect (session * sess, int sendquit, int err)
 	serv->motd_skipped = FALSE;
 	serv->no_login = FALSE;
 	serv->servername[0] = 0;
-	if (serv->networkname)
-	{
-		free (serv->networkname);
-		serv->networkname = NULL;
-	}
 	serv->lag_sent = 0;
 
 	notify_cleanup ();
@@ -1095,12 +1113,52 @@ traverse_socks5 (int sok, char *serverAddr, int port)
 
 	sc1.version = 5;
 	sc1.nmethods = 1;
-	sc1.method = 0;
+	if ( prefs.proxy_auth )
+	{
+		if ( !prefs.proxy_user[0] || !prefs.proxy_pass[0] )
+		{
+			return 1;
+		}
+		sc1.method = 2;  /* Username/Password Authentication (UPA) */
+	}
+	else
+	{
+		sc1.method = 0;  /* NO Authentication */
+	}
 	send (sok, (char *) &sc1, 3, 0);
 	if (recv (sok, buf, 2, 0) != 2)
 		return 1;
-	if (buf[0] != 5 && buf[1] != 0)
-		return 1;
+
+	if ( prefs.proxy_auth )
+	{
+		int len_u=0, len_p=0;
+
+		/* authentication sub-negotiation (RFC1929) */
+		if ( buf[0] != 5 || buf[1] != 2 )  /* UPA not supported by server */
+			return 1;
+
+		memset (buf, 0, sizeof(buf));
+
+		/* form the UPA request */
+		len_u = strlen (prefs.proxy_user);
+		len_p = strlen (prefs.proxy_pass);
+		buf[0] = 1;
+		buf[1] = len_u;
+		memcpy (buf + 2, prefs.proxy_user, len_u);
+		buf[2 + len_u] = len_p;
+		memcpy (buf + 3 + len_u, prefs.proxy_pass, len_p);
+
+		send (sok, buf, 3 + len_u + len_p, 0);
+		if ( recv (sok, buf, 2, 0) != 2 )
+			return 1;
+		if ( buf[1] != 0 )
+			return 1; /* UPA failed! */
+	}
+	else
+	{
+		if (buf[0] != 5 || buf[1] != 0)
+			return 1;
+	}
 
 	addrlen = strlen (serverAddr);
 	packetlen = 4 + 1 + addrlen + 2;
@@ -1197,7 +1255,35 @@ base64_encode (char *to, char *from, unsigned int len)
 }
 
 static int
-traverse_http (int sok, char *serverAddr, int port)
+http_read_line (int print_fd, int sok, char *buf, int len)
+{
+#ifdef WIN32
+	/* make sure waitline() uses recv() or it'll fail on win32 */
+	len = waitline (sok, buf, len, FALSE);
+#else
+	len = waitline (sok, buf, len, TRUE);
+#endif
+	if (len >= 1)
+	{
+		/* print the message out (send it to the parent process) */
+		write (print_fd, "0\n", 2);
+
+		if (buf[len-1] == '\r')
+		{
+			buf[len-1] = '\n';
+			write (print_fd, buf, len);
+		} else
+		{
+			write (print_fd, buf, len);
+			write (print_fd, "\n", 1);
+		}
+	}
+
+	return len;
+}
+
+static int
+traverse_http (int print_fd, int sok, char *serverAddr, int port)
 {
 	char buf[256];
 	char auth_data[128];
@@ -1216,27 +1302,24 @@ traverse_http (int sok, char *serverAddr, int port)
 	n += snprintf (buf+n, sizeof (buf)-n, "\r\n");
 	send (sok, buf, n, 0);
 
-	/* dont eat this, it's actually informative to display */
-#if 0
-	waitline (sok, buf, sizeof (buf)); /* FIXME: win32 cant read() sok */
+	n = http_read_line (print_fd, sok, buf, sizeof (buf));
 	/* "HTTP/1.0 200 OK" */
-	if (strlen (buf) < 12)
+	if (n < 12)
 		return 1;
 	if (memcmp (buf, "HTTP/", 5) || memcmp (buf + 9, "200", 3))
 		return 1;
-	for (;;)
+	while (1)
 	{
 		/* read until blank line */
-		waitline (sok, buf, sizeof (buf));
-		if (!buf[0] || (buf[0] == '\r' && !buf[1]))
+		n = http_read_line (print_fd, sok, buf, sizeof (buf));
+		if (n < 1 || (n == 1 && buf[0] == '\n'))
 			break;
 	}
-#endif
 	return 0;
 }
 
 static int
-traverse_proxy (int sok, char *ip, int port)
+traverse_proxy (int print_fd, int sok, char *ip, int port)
 {
 	switch (prefs.proxy_type)
 	{
@@ -1247,7 +1330,7 @@ traverse_proxy (int sok, char *ip, int port)
 	case 3:
 		return traverse_socks5 (sok, ip, port);
 	case 4:
-		return traverse_http (sok, ip, port);
+		return traverse_http (print_fd, sok, ip, port);
 	}
 
 	return 1;
@@ -1343,7 +1426,7 @@ server_child (server * serv)
 		/* connect succeeded */
 		if (proxy_ip)
 		{
-			switch (traverse_proxy (sok, proxy_ip, port))
+			switch (traverse_proxy (serv->childwrite, sok, proxy_ip, port))
 			{
 			case 0:	/* success */
 				snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
@@ -1425,7 +1508,7 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	safe_strcpy (serv->servername, hostname, sizeof (serv->servername));
 	safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
 
-	set_server_defaults (serv);
+	server_set_defaults (serv);
 	serv->connecting = TRUE;
 	serv->port = port;
 	serv->no_login = no_login;
@@ -1471,7 +1554,8 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 		fe_input_add (serv->childread, FIA_READ|FIA_FD, server_read_child, serv);
 }
 
-void server_fill_her_up (server *serv)
+void
+server_fill_her_up (server *serv)
 {
 	serv->connect = server_connect;
 	serv->disconnect = server_disconnect;
@@ -1481,3 +1565,96 @@ void server_fill_her_up (server *serv)
 
 	proto_fill_her_up (serv);
 }
+
+void
+server_set_encoding (server *serv, char *new_encoding)
+{
+	char *space;
+
+	if (serv->encoding)
+	{
+		free (serv->encoding);
+		/* can be left as NULL to indicate system encoding */
+		serv->encoding = NULL;
+		serv->using_cp1255 = FALSE;
+	}
+
+	if (new_encoding)
+	{
+		serv->encoding = strdup (new_encoding);
+		/* the serverlist GUI might have added a space 
+			and short description - remove it. */
+		space = strchr (serv->encoding, ' ');
+		if (space)
+			space[0] = 0;
+
+		/* server_inline() uses this flag */
+		if (!strcasecmp (serv->encoding, "CP1255") ||
+			 !strcasecmp (serv->encoding, "WINDOWS-1255"))
+			serv->using_cp1255 = TRUE;
+	}
+}
+
+server *
+server_new (void)
+{
+	static int id = 0;
+	server *serv;
+
+	serv = malloc (sizeof (struct server));
+	memset (serv, 0, sizeof (struct server));
+
+	/* use server.c and proto-irc.c functions */
+	server_fill_her_up (serv);
+
+	serv->id = id++;
+	serv->sok = -1;
+	strcpy (serv->nick, prefs.nick1);
+	server_set_defaults (serv);
+
+	serv_list = g_slist_prepend (serv_list, serv);
+
+	fe_new_server (serv);
+
+	return serv;
+}
+
+void
+server_set_defaults (server *serv)
+{
+	if (serv->chantypes)
+		free (serv->chantypes);
+	if (serv->chanmodes)
+		free (serv->chanmodes);
+	if (serv->nick_prefixes)
+		free (serv->nick_prefixes);
+	if (serv->nick_modes)
+		free (serv->nick_modes);
+
+	serv->chantypes = strdup ("#&!+");
+	serv->chanmodes = strdup ("beI,k,l");
+	serv->nick_prefixes = strdup ("@%+");
+	serv->nick_modes = strdup ("ohv");
+
+	serv->nickcount = 1;
+	serv->nickservtype = 0;
+	serv->end_of_motd = FALSE;
+	serv->is_away = FALSE;
+	serv->supports_watch = FALSE;
+	serv->bad_prefix = FALSE;
+	serv->use_who = TRUE;
+	serv->have_idmsg = FALSE;
+	serv->have_except = FALSE;
+}
+
+char *
+server_get_network (server *serv, gboolean fallback)
+{
+	if (serv->network)
+		return ((ircnet *)serv->network)->name;
+
+	if (fallback)
+		return serv->servername;
+
+	return NULL;
+}
