@@ -14,9 +14,8 @@
 struct text_event
 {
 	char *name;
-	char **help;
+	char * const *help;
 	int num_args;
-	char *sound;
 	char *def;
 };
 
@@ -34,5 +33,12 @@ void text_emit (int index, session *sess, char *a, char *b, char *c, char *d);
 int text_emit_by_name (char *name, session *sess, char *a, char *b, char *c, char *d);
 char *text_validate (char **text, int *len);
 int get_stamp_str (char *fmt, time_t tim, char **ret);
+void format_event (session *sess, int index, char **args, char *o, int sizeofo);
+
+void sound_play (const char *file);
+void sound_play_event (int i);
+void sound_beep (session *);
+void sound_load ();
+void sound_save ();
 
 #endif
